@@ -48,7 +48,6 @@ extern fontinfo *fonts;
 extern short fontcount;
 extern short attachto;
 extern char  _STR[150], _STR2[250];
-extern stringpart *labelbuf;
 extern int number_colors;
 
 extern double atan2();
@@ -525,7 +524,7 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
    for (tgen = topobject->plist; tgen < topobject->plist + topobject->parts; tgen++) {
       if (IS_LABEL(*tgen)) {
          tlab = TOLABEL(tgen);
-	 if (tlab->pin == False) continue;
+	 if (tlab->pin != LOCAL) continue;
 	 else if (tlab == curlabel) continue;  /* Don't count self! */
          else if (!stringcomp(tlab->string, curstring)) return tlab;  
       }
@@ -542,9 +541,9 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
 void labeltext(int keypressed, char *clientdata)
 {
    labelptr curlabel;
-   stringpart *curpos;
+   stringpart *curpos, *labelbuf;
    int locpos;
-   Boolean do_redraw = False;
+   Boolean do_redraw = False, dp = FALSE;
    short tmplength, tmpheight, cfont;
    TextExtents tmpext;
 
@@ -560,7 +559,8 @@ void labeltext(int keypressed, char *clientdata)
 
    UDrawTLine(curlabel);
 
-   if (isbound(keypressed, XCF_Text_Delete) && textpos > 1) {
+   if ((textpos > 1) && (isbound(keypressed, XCF_Text_Delete) ||
+		(dp = isbound(keypressed, XCF_Text_Delete_Param)))) {
       int curloc, strpos;
       stringpart *strptr;
 
@@ -578,12 +578,19 @@ void labeltext(int keypressed, char *clientdata)
 	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
 	 }
 
-         /* Don't delete any parameter boundaries---must use	*/
-         /* "unparameterize" command for that.			*/
+         /* When encountering parameter boundaries---use "unparameterize" */
+	 /* command or XCF_Text_Delete_Param (forced deletion)		  */
 
 	 else if (strptr != NULL) {
 	    if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
 	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
+	    else if (dp && (strptr->type == PARAM_END)) {
+	       while(strptr->type != PARAM_START) {
+		  strptr = findstringpart(--strpos, &curloc, curlabel->string,
+			 areastruct.topinstance);
+	       }
+	       unmakeparam(curlabel, strptr);
+	    }
 	    else
 	       textpos++;
 	 }
@@ -608,26 +615,41 @@ void labeltext(int keypressed, char *clientdata)
 
       if (hasstuff && (eventmode != TEXT3_MODE && eventmode != CATTEXT_MODE)) {
 	 topobject->parts++;
+         register_for_undo(XCF_Text, UNDO_DONE, areastruct.topinstance,
+                    curlabel);
+
 	 incr_changes(topobject);
 	 invalidate_netlist(topobject);
 
       }
-      else if (!hasstuff && (eventmode == TEXT3_MODE)) topobject->parts--;
+      else if (!hasstuff && (eventmode == TEXT3_MODE)) {
+	 if (areastruct.editpart < topobject->parts) {
+	    short *sptr = allocselect();
+	    *sptr = areastruct.editpart;
+
+	    /* Force the "delete" undo record to be a continuation of	*/
+	    /* the undo series containing the edit.  That way, "undo"	*/
+	    /* does not generate a label with null text.		*/
+
+	    xobjs.undostack->idx = -xobjs.undostack->idx;
+	    standard_element_delete(NORMAL);
+	 }
+	 else {
+	    /* Label had just been created; just delete it w/o undo */
+	    freelabel(curlabel->string);
+	    free(curlabel);
+	    topobject->parts--;
+	 }
+      }
 
       if ((!hasstuff) && (eventmode == CATTEXT_MODE)) {  /* can't have null labels! */ 
-	  freelabel(curlabel->string);
-	  curlabel->string = stringcopyback(labelbuf, areastruct.topinstance);
-	  freelabel(labelbuf);
-	  labelbuf = NULL;
-	  resolveparams(areastruct.topinstance);
+	  undo_action();
           XcSetFunction(GXcopy);
 	  redrawtext(curlabel);
 	  Wprintf("Object must have a name!");
 	  eventmode = CATALOG_MODE;
       }
       else if (!hasstuff) {
-	  freelabel(curlabel->string);
-	  free(curlabel);
 	  eventmode = NORMAL_MODE;
       }
       else if (eventmode == CATTEXT_MODE) {
@@ -666,7 +688,7 @@ void labeltext(int keypressed, char *clientdata)
 
       setdefaultfontmarks();
       setcolormark(areastruct.color);
-      if (labelbuf != NULL) {
+      if ((labelbuf = get_original_string(curlabel)) != NULL) {
 
 	 /* If the original label (before modification) is a pin in a	*/
 	 /* schematic/symbol with a matching symbol/schematic, and the	*/
@@ -687,8 +709,6 @@ void labeltext(int keypressed, char *clientdata)
 	    }
 	 }
 	
-	 freelabel(labelbuf);
-	 labelbuf = NULL;
 	 resolveparams(areastruct.topinstance);
          updateinstparam(topobject);
 	 setobjecttype(topobject);
@@ -957,6 +977,7 @@ void rejustify(short mode)
       Wprintf("No labels chosen to rejustify");
    else if (changed) {
       pwriteback(areastruct.topinstance);
+      calcbbox(areastruct.topinstance);
       incr_changes(topobject);
    }
 }
@@ -1040,6 +1061,8 @@ void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*--------------------------*/
@@ -1177,6 +1200,8 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*----------------------*/
@@ -1268,6 +1293,8 @@ void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*----------------------------------*/
@@ -1294,6 +1321,8 @@ void trackwire(xcWidget w, caddr_t clientdata, caddr_t calldata)
       areastruct.save.y = newpos.y;
       printpos(newpos.x, newpos.y);
    }
+
+   flusharea();
 }
 
 /*--------------------------*/
@@ -1468,6 +1497,8 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*-------------------------------------------------*/
@@ -1552,19 +1583,13 @@ void reversefpoints(XfPoint *plist, short number)
 /*	add = 1 if plist has (parts + 1) elements		*/
 /*--------------------------------------------------------------*/
 
-void freeparts(short *selectobj, short add)
+void freepathparts(short *selectobj, short add)
 {
    genericptr *oldelem = topobject->plist + (*selectobj);
    switch(ELEMENTTYPE(*oldelem)) {
       case POLYGON:
 	 free((TOPOLY(oldelem))->points);
 	 break;
-      case PATH:
-	 free((TOPATH(oldelem))->plist);
-	 break;
-      case LABEL:
-	 free((TOLABEL(oldelem))->string);
-	 break;
    }
    free(*oldelem);
    removep(selectobj, add);
@@ -1628,7 +1653,7 @@ void unjoin()
 
          /* remove the path object and revise the selectlist */
 
-         freeparts(selectobj, 0);
+         freepathparts(selectobj, 0);
          reviseselect(areastruct.selectlist, areastruct.selects, selectobj);
       }
    }
@@ -1855,7 +1880,7 @@ void join()
 
       for (sptr = scount; sptr < scount + numpolys; sptr++) {
 	 easydraw(*sptr, DOFORALL);
-	 freeparts(sptr, 1);
+	 freepathparts(sptr, 1);
 	 /* revise the list of path elements */
 	 for (sptr2 = sptr + 1; sptr2 < scount + numpolys; sptr2++)
 	    if (*sptr2 > *sptr) (*sptr2)--;
