@@ -11,6 +11,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <limits.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -291,23 +292,44 @@ float findsplinemin(splineptr thespline, XPoint *upoint)
 /* Find closest point of a polygon to the cursor			      */
 /*----------------------------------------------------------------------------*/
 
-short closepoint(polyptr curpoly, XPoint *cursloc)
+short closepointdistance(polyptr curpoly, XPoint *cursloc, short *mindist)
 {
-   short curdist, mindist;
+   short curdist;
    XPoint *curpt, *savept; 
 
    curpt = savept = curpoly->points;
-   mindist = wirelength(curpt, cursloc);
+   *mindist = wirelength(curpt, cursloc);
    while (++curpt < curpoly->points + curpoly->number) {
       curdist = wirelength(curpt, cursloc);
-      if (curdist < mindist) {
-         mindist = curdist;
+      if (curdist < *mindist) {
+         *mindist = curdist;
          savept = curpt;
       }
    }
    return (short)(savept - curpoly->points);
 }
 
+/*----------------------------------------------------------------------------*/
+/* Find closest point of a polygon to the cursor			      */
+/*----------------------------------------------------------------------------*/
+
+short closepoint(polyptr curpoly, XPoint *cursloc)
+{
+   short mindist;
+   return closepointdistance(curpoly, cursloc, &mindist);
+}
+
+/*----------------------------------------------------------------------------*/
+/* Find the distance to the closest point of a polygon to the cursor	      */
+/*----------------------------------------------------------------------------*/
+
+short closedistance(polyptr curpoly, XPoint *cursloc)
+{
+   short mindist;
+   closepointdistance(curpoly, cursloc, &mindist);
+   return mindist;
+}
+
 /*----------------------------------------------------------------------------*/
 /* Coordinate system transformations 					      */
 /*----------------------------------------------------------------------------*/
@@ -403,11 +425,9 @@ float UTopScale()
 
 /*------------------------------------------------------------------------*/
 
-short UTopTransScale(float length)
+float UTopTransScale(float length)
 {
-   /* 0.5 gives nearest int but unfortunately gives odd results. . .*/
-  
-   return (short)(length * UTopScale() + 0.45);
+   return (float)(length * UTopScale());
 }
 
 /*----------------------------------------------------------------------*/
@@ -421,8 +441,16 @@ XPoint UGetCursor()
    u_int   nullui;
    XPoint newpos;
  
+#ifdef TCL_WRAPPER
+   /* Don't use areastruct.areawin;  if called from inside an object	*/
+   /* (e.g., "here" in a Tcl expression), areastruct.areawin will be	*/
+   /* an off-screen pixmap, and cause a crash.				*/
+   XQueryPointer(dpy, Tk_WindowId(areastruct.area), &nullwin, &nullwin,
+	&nullint, &nullint, &xpos, &ypos, &nullui);
+#else
    XQueryPointer(dpy, areastruct.areawin, &nullwin, &nullwin, &nullint,
 	&nullint, &xpos, &ypos, &nullui);
+#endif
 
    newpos.x = xpos;
    newpos.y = ypos;
@@ -849,14 +877,17 @@ void calcextents(genericptr *bboxgen, short *llx, short *lly,
 /* Calculate the bounding box of an object instance			*/
 /*----------------------------------------------------------------------*/
 
-void objinstbbox(objinstptr obbox, XPoint *npoints)
+void objinstbbox(objinstptr obbox, XPoint *npoints, Boolean expand)
 {
    XPoint points[4];
+   int extend = (expand) ? 4 : 0;
 
-   points[0].x = points[1].x = obbox->bbox.lowerleft.x;
-   points[1].y = points[2].y = obbox->bbox.lowerleft.y + obbox->bbox.height;
-   points[2].x = points[3].x = obbox->bbox.lowerleft.x + obbox->bbox.width;
-   points[0].y = points[3].y = obbox->bbox.lowerleft.y;
+   points[0].x = points[1].x = obbox->bbox.lowerleft.x - extend;
+   points[1].y = points[2].y = obbox->bbox.lowerleft.y + obbox->bbox.height
+		+ extend;
+   points[2].x = points[3].x = obbox->bbox.lowerleft.x + obbox->bbox.width
+		+ extend;
+   points[0].y = points[3].y = obbox->bbox.lowerleft.y - extend;
 
    UTransformPoints(points, npoints, 4, obbox->position,
 		 obbox->scale, obbox->rotation); 
@@ -893,6 +924,26 @@ void labelbbox(labelptr labox, XPoint *npoints, objinstptr callinst)
 		labox->scale, labox->rotation); 
 }
 
+/*----------------------------------------------------------------------*/
+/* Calculate the bounding box of a graphic image			*/
+/*----------------------------------------------------------------------*/
+
+void graphicbbox(graphicptr gp, XPoint *npoints)
+{
+   XPoint points[4];
+   int hw = gp->source->width >> 1;
+   int hh = gp->source->height >> 1;
+
+   points[1].x = points[2].x = hw;
+   points[0].x = points[3].x = -hw;
+
+   points[0].y = points[1].y = -hh;
+   points[2].y = points[3].y = hh;
+
+   UTransformPoints(points, npoints, 4, gp->position,
+		gp->scale, gp->rotation); 
+}
+
 /*--------------------------------------------------------------*/
 /* Wrapper for single call to calcbboxsingle() in the netlister */
 /*--------------------------------------------------------------*/
@@ -922,7 +973,7 @@ void calcbboxsingle(genericptr *bboxgen, objinstptr thisinst,
    switch(ELEMENTTYPE(*bboxgen)) {
 
       case(OBJINST):
-	 objinstbbox(TOOBJINST(bboxgen), npoints);
+	 objinstbbox(TOOBJINST(bboxgen), npoints, False);
 
          for (j = 0; j < 4; j++) {
             bboxcalc(npoints[j].x, llx, urx);
@@ -931,21 +982,28 @@ void calcbboxsingle(genericptr *bboxgen, objinstptr thisinst,
 	 break;
 
       case(LABEL):
-	 {
-	    /* because a pin is offset from its position point, include */
-	    /* that point in the bounding box.				*/
+	 /* because a pin is offset from its position point, include */
+	 /* that point in the bounding box.				*/
 
-	    if (TOLABEL(bboxgen)->pin) {
-               bboxcalc(TOLABEL(bboxgen)->position.x, llx, urx);
-               bboxcalc(TOLABEL(bboxgen)->position.y, lly, ury);
-	    }
-	    labelbbox(TOLABEL(bboxgen), npoints, thisinst);
+	 if (TOLABEL(bboxgen)->pin) {
+            bboxcalc(TOLABEL(bboxgen)->position.x, llx, urx);
+            bboxcalc(TOLABEL(bboxgen)->position.y, lly, ury);
+	 }
+	 labelbbox(TOLABEL(bboxgen), npoints, thisinst);
 
-            for (j = 0; j < 4; j++) {
-               bboxcalc(npoints[j].x, llx, urx);
-               bboxcalc(npoints[j].y, lly, ury);
-            }
-         } break;
+         for (j = 0; j < 4; j++) {
+            bboxcalc(npoints[j].x, llx, urx);
+            bboxcalc(npoints[j].y, lly, ury);
+         }
+         break;
+
+      case(GRAPHIC):
+	 graphicbbox(TOGRAPHIC(bboxgen), npoints);
+         for (j = 0; j < 4; j++) {
+            bboxcalc(npoints[j].x, llx, urx);
+            bboxcalc(npoints[j].y, lly, ury);
+         }
+	 break;
 
       case(PATH): {
 	 genericptr *pathc;
@@ -1326,6 +1384,7 @@ void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 	 eventmode == FONTCAT2_MODE) {
 
       centerview(areastruct.topinstance);
+      invalidate_graphics(topobject);
       areastruct.lastbackground = NULL;
       renderbackground();
       refresh(NULL, NULL, NULL);
@@ -1343,7 +1402,7 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
    UTransformbyCTM(DCTM, pt1, &newpt1, 1);
    UTransformbyCTM(DCTM, pt2, &newpt2, 1);
 
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc,
 		newpt1.x, newpt1.y, newpt2.x, newpt2.y); 
 } 
 
@@ -1351,10 +1410,9 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
 
 void UDrawLine(XPoint *pt1, XPoint *pt2)
 {
-   short tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
+   float tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
 
-   XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ? (int)tmpwidth : 
-	0, LineSolid, CapRound, JoinBevel);
+   SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound, JoinBevel);
    UDrawSimpleLine(pt1, pt2); 
 } 
 
@@ -1377,7 +1435,7 @@ void UDrawCircle(XPoint *upt, u_char which)
    XPoint wpt;
 
    user_to_window(*upt, &wpt);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
 
    switch(which) {
       case P_POSITION_X:
@@ -1405,10 +1463,10 @@ void UDrawCircle(XPoint *upt, u_char which)
 
 void UDrawXAt(XPoint *wpt)
 {
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x - 3,
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x - 3,
 		wpt->y - 3, wpt->x + 3, wpt->y + 3);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x + 3,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x + 3,
 		wpt->y - 3, wpt->x - 3, wpt->y + 3);
 }
 
@@ -1545,8 +1603,8 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    UPreMultCTM(DCTM, curlabel->position, curlabel->scale, curlabel->rotation);
    tmpjust = flipadjust(curlabel->justify);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
 
    tmpext = ULength(curlabel->string, areastruct.topinstance, 0.0, tpos, NULL);
    xdist = tmpext.width;
@@ -1595,23 +1653,23 @@ void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
 
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   XSetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
 
    user_to_window(cpt, &upt);
    user_to_window(opt, &vpt);
 
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineOnOffDash, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, vpt.x, vpt.y, upt.x, upt.y);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineOnOffDash, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, vpt.x, vpt.y, upt.x, upt.y);
 
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x - 3, upt.y - 3,
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x - 3, upt.y - 3,
 		upt.x + 3, upt.y + 3);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x + 3, upt.y - 3,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x + 3, upt.y - 3,
 		upt.x - 3, upt.y + 3);
 
-   XSetFunction(dpy, areastruct.gc, areastruct.gctype);
-   XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
+   SetFunction(dpy, areastruct.gc, areastruct.gctype);
+   SetForeground(dpy, areastruct.gc, areastruct.gccolor);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1623,20 +1681,79 @@ void UDrawBox(XPoint origin, XPoint corner)
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
 
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
+/*----------------------------------------------------------------------*/
+/* Draw a box indicating the dimensions of the edit element that most	*/
+/* closely reach the position "corner".					*/
+/*----------------------------------------------------------------------*/
+
+void UDrawRescaleBox(XPoint *corner)
+{
+   XPoint 	origpoints[5], newpoints[5];
+   genericptr	*rgen = EDITPART;
+   float	savescale, newscale, f1, f2;
+   long		mindist, testdist;
+   labelptr	rlab;
+   int		i, idx;
+
+   switch(ELEMENTTYPE(*rgen)) {
+      case LABEL:
+	 rlab = TOLABEL(rgen);
+	 labelbbox(rlab, origpoints, areastruct.topinstance);
+	 origpoints[5] = origpoints[0];
+	 savescale = rlab->scale;
+	 rlab->scale *= 2.0;
+	 labelbbox(rlab, newpoints, areastruct.topinstance);
+	 rlab->scale = savescale;
+	 newpoints[5] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist) {
+	       mindist = testdist;
+	       idx = i;
+	    }
+	 }
+	 testdist = finddist(&origpoints[idx], &origpoints[idx + 1], corner);
+	 f2 = (float)sqrt((double)mindist);
+	 f1 = (float)sqrt((double)testdist);
+	 if (f1 == f2) break;
+	 newscale = (f2 - 2.0 * f1) / (f2 - f1);
+	 newscale *= rlab->scale;
+	 if (newscale > 20) newscale = 20;	/* temporary---set limits */
+	 if (newscale < 0.1) newscale = 0.1;
+	 rlab->scale = newscale;
+	 labelbbox(rlab, newpoints, areastruct.topinstance);
+	 rlab->scale = savescale;
+         break;
+      case GRAPHIC:
+         break;
+      case OBJINST:
+         break;
+      /* Note:  should be able to handle other elements, as well */
+   }
+
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   
+   UTransformbyCTM(DCTM, newpoints, origpoints, 4);
+   strokepath(origpoints, 4, 0, 1);
+}
+
 /*-------------------------------------------------------------------------*/
 void UDrawBBox()
 {
@@ -1656,14 +1773,14 @@ void UDrawBBox()
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   XSetForeground(dpy, areastruct.gc, BBOXCOLOR);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   SetForeground(dpy, areastruct.gc, BBOXCOLOR);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
@@ -1675,50 +1792,50 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
 {
    char         solidpart;
    char         dashstring[3];
-   short        tmpwidth;
+   float        tmpwidth;
+   short	minwidth;
 
-   tmpwidth = max(1, UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth *
-	width));
+   tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth * width);
+   minwidth = max(1, (short)tmpwidth);
 
    if (style & FILLED || (!(style & FILLED) && style & OPAQUE)) {
       if ((style & FILLSOLID) == FILLSOLID)
-         XSetFillStyle(dpy, areastruct.gc, FillSolid);
+         SetFillStyle(dpy, areastruct.gc, FillSolid);
       else if (!(style & FILLED)) {
-         XSetFillStyle(dpy, areastruct.gc, FillOpaqueStippled); 
-	 XSetStipple(dpy, areastruct.gc, STIPPLE[7]);
+         SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled); 
+	 SetStipple(dpy, areastruct.gc, 7);
       }
       else {
 	 if (style & OPAQUE)
-            XSetFillStyle(dpy, areastruct.gc, FillOpaqueStippled);
+            SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled);
 	 else
-            XSetFillStyle(dpy, areastruct.gc, FillStippled);
-         XSetStipple(dpy, areastruct.gc, STIPPLE[ (style &
-                FILLSOLID) >> 5] );
+            SetFillStyle(dpy, areastruct.gc, FillStippled);
+         SetStipple(dpy, areastruct.gc, ((style & FILLSOLID) >> 5));
       }
-      XFillPolygon(dpy, areastruct.areawin, areastruct.gc, pathlist, number, Nonconvex,
+      FillPolygon(dpy, areastruct.areawin, areastruct.gc, pathlist, number, Nonconvex,
 		CoordModeOrigin);
       /* return to original state */
-      XSetFillStyle(dpy, areastruct.gc, FillSolid);
+      SetFillStyle(dpy, areastruct.gc, FillSolid);
    }
    if (!(style & NOBORDER)) {
       /* set up dots or dashes */
-      if (style & DASHED) solidpart = (char)(4 * tmpwidth);
-      else if (style & DOTTED) solidpart = (char)tmpwidth;
-      sprintf(dashstring, "%c%c", solidpart, (char)(4 * tmpwidth));
+      if (style & DASHED) solidpart = (char)(4 * minwidth);
+      else if (style & DOTTED) solidpart = (char)minwidth;
+      sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
       if (style & (DASHED | DOTTED)) {
-         XSetDashes(dpy, areastruct.gc, 0, dashstring, 2);
-         XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ?
-            (int)tmpwidth : 0, LineOnOffDash, CapButt, JoinBevel);
+         SetDashes(dpy, areastruct.gc, 0, dashstring, 2);
+         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineOnOffDash,
+		CapButt, JoinBevel);
       }
       else
-         XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ?
-            (int)tmpwidth : 0, LineSolid, CapRound, JoinBevel);
+         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid,
+		CapRound, JoinBevel);
 
       /* draw the spline and close off if so specified */
-      XDrawLines(dpy, areastruct.areawin, areastruct.gc, pathlist,
+      DrawLines(dpy, areastruct.areawin, areastruct.gc, pathlist,
 		number, CoordModeOrigin);
       if (!(style & UNCLOSED))
-         XDrawLine(dpy, areastruct.areawin, areastruct.gc, pathlist[0].x,
+         DrawLine(dpy, areastruct.areawin, areastruct.gc, pathlist[0].x,
 		pathlist[0].y, pathlist[number - 1].x, pathlist[number - 1].y);
    }
 }
@@ -1826,7 +1943,7 @@ void UDrawPath(pathptr thepath)
 void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr *stack)
 {
    genericptr	*areagen;
-   short	tmpwidth;
+   float	tmpwidth;
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    int		thispart;
@@ -1864,8 +1981,8 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
      /* draw all of the elements */
    
      tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
-     XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ? (int)tmpwidth :
-		0, LineSolid, CapRound, JoinBevel);
+     SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound,
+		JoinBevel);
 
      /* guard against plist being regenerated during a redraw by the	*/
      /* expression parameter mechanism (should that be prohibited?)	*/
@@ -1901,6 +2018,10 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	  case(PATH):
 	     UDrawPath(TOPATH(areagen));
 	     break;
+
+	  case(GRAPHIC):
+	     UDrawGraphic(TOGRAPHIC(areagen));
+	     break;
    
           case(OBJINST):
 	     if (areastruct.editinplace && stack && (TOOBJINST(areagen)
@@ -1926,6 +2047,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 			TOLABEL(areagen)->pin != INFO && areastruct.pinpointon)
 		UDrawXDown(TOLABEL(areagen));
 	     break;
+
        }
      }
 
