@@ -500,7 +500,7 @@ extern void setgridtype(char *);
 extern void getgridtype(xcWidget, pointertype, caddr_t);
 extern void newlibrary(xcWidget, caddr_t, caddr_t);
 extern int findemptylib();
-extern int createlibrary();
+extern int createlibrary(Boolean);
 extern void newpagemenu(xcWidget, pointertype, caddr_t);
 extern void makepagebutton();
 extern void renamepage(short);
@@ -567,6 +567,7 @@ extern void createnets(objinstptr);
 extern void destroynets();
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
+extern LabellistPtr geninfolist(objectptr, objinstptr, char *);
 extern void gennetlist(objinstptr);
 extern void gencalls(objectptr);
 extern void search_on_siblings(objinstptr, objinstptr, pushlistptr,
@@ -597,12 +598,13 @@ extern Genericlist *nametonet(objectptr, objinstptr, char *);
 extern Genericlist *new_tmp_pin(objectptr, XPoint *, char *, char *, Genericlist *);
 extern Genericlist *make_tmp_pin(objectptr, objinstptr, XPoint *, Genericlist *);
 extern void resolve_devnames(objectptr);
-extern void resolve_devindex(objectptr, Boolean);
+extern void resolve_indices(objectptr, Boolean);
+extern void clear_indices(objectptr);
 extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
 extern void writeglobals(objectptr, FILE *);
-extern void writehierarchy(objectptr, CalllistPtr, FILE *);
+extern void writehierarchy(objectptr, CalllistPtr, FILE *, char *);
 extern void writenet(objectptr, char *, char *);
 extern Boolean writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(struct Ptab *, FILE *);
@@ -691,6 +693,7 @@ extern Imagedata *addnewimage(char *, int, int);
 extern graphicptr new_graphic(objinstptr, char *, int, int);
 extern void invalidate_graphics(objectptr);
 extern void freegraphic(graphicptr);
+extern short *collect_graphics(short *);
 
 /* from render.c: */
 
@@ -717,7 +720,7 @@ extern Boolean render_client();
 
 /* from schema.c: */
 
-extern objectptr NameToPageObject(char *, objinstptr *);
+extern objectptr NameToPageObject(char *, objinstptr *, int *);
 extern objectptr NameToObject(char *, objinstptr *, Boolean);
 extern void callwritenet(xcWidget, pointertype, caddr_t);
 extern void doxschema(xcWidget, pointertype, caddr_t);
@@ -779,6 +782,7 @@ extern stringpart *splitstring(int, stringpart **, objinstptr);
 extern stringpart *mergestring(stringpart *);
 extern stringpart *linkstring(objinstptr, stringpart *, Boolean);
 extern int findcurfont(int, stringpart *, objinstptr);
+extern stringpart *findtextinstring(char *, int *, stringpart *, objinstptr);
 extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
