@@ -171,7 +171,7 @@ short execcommand(short pflags, char *cmdptr)
 	 libnum = LIBRARY;
       else {
 	 if (libnum >= xobjs.numlibs)
-	    libnum = createlibrary();
+	    libnum = createlibrary(FALSE);
 	 else
 	    libnum += LIBRARY - 1;
       }
