@@ -62,6 +62,7 @@ extern int xctcl_instance();
 extern int xctcl_spline();
 extern int xctcl_path();
 extern int xctcl_arc();
+extern int xctcl_graphic();
 extern int xctcl_object();
 extern int xctcl_here();
 extern int xctcl_start();
@@ -116,6 +117,7 @@ static cmdstruct xc_commands[] =
    {"spline", (void *)xctcl_spline},
    {"path", (void *)xctcl_path},
    {"arc", (void *)xctcl_arc},
+   {"graphic", (void *)xctcl_graphic},
    {"object", (void *)xctcl_object},
    {"here", (void *)xctcl_here},
    {"netlist", (void *)xctcl_netlist},
@@ -199,7 +201,10 @@ Xcircuit_Init(Tcl_Interp *interp)
    if (cadhome == NULL) cadhome = CAD_DIR;
    Tcl_SetVar(interp, "CAD_HOME", cadhome, TCL_GLOBAL_ONLY);
 
-   /* Set $XCIRCUIT_VERSION as a Tcl variable */
+   /* Set $XCIRCUIT_VERSION and $XCIRCUIT_REVISION as Tcl variables */
+
+   sprintf(version_string, "%d", PROG_REVISION);
+   Tcl_SetVar(interp, "XCIRCUIT_REVISION", version_string, TCL_GLOBAL_ONLY);
 
    sprintf(version_string, "%g", PROG_VERSION);
    Tcl_SetVar(interp, "XCIRCUIT_VERSION", version_string, TCL_GLOBAL_ONLY);
